/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

extern char *cwd;
extern char *tmp_dir;
extern char *prog_name;

FILE        *urandom;
BGEN_ALGO    bgen_algo;

/* Set bgen_algo */
void
set_bgen(BGEN_ALGO algo)
{
    bgen_algo = algo;
}

/* Generate a single byte */
int
genbyte(bgen_t *byte)
{
    enum {
      OK,
      FAILURE
    };

    if(bgen_algo == SSL) {
      if(RAND_bytes(byte, 1) <= 0) {
        return(FAILURE);
      }
    } else if(bgen_algo == URANDOM) {
      if(!fread(byte, sizeof(bgen_t), 1, urandom)) {
        return(FAILURE);
      }
    }
    return(OK);
}

/* Generate a key <length> bytes long */
void
genkey(FILE *key, int length)
{
    int     i;
    bgen_t  byte;

    if(bgen_algo == URANDOM) {
      if((urandom = fopen("/dev/urandom", "r")) == NULL) {
        Err(OPEN, "/dev/urandom");
      }
    }

    for(i = 0; i < length; i++) {
      if(genbyte(&byte)) {
        Err(BYTE_GEN, NULL);
      }
      Wbyte(&byte, key);
    }

    if(bgen_algo == URANDOM) {
      Fclose(urandom);
    }
    rewind(key);
    return;
}
