/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

extern char *cwd;
extern char *tmp_dir;
extern char *prog_name;

/* General error function */
void
Err(err_code code, const char *arg)
{
    switch(code) {
      case OPEN:
        Errx("Could not open `%s'\n",
          arg);
        break;
      case READ:
        Errx("Could not read byte");
        break;
      case WRITE:
        Errx("Could not write byte");
        break;
      case RENAME:
        Warn("Could not rename `%s'",
          arg);
        break;
      case DIROPEN:
        Errx("Error opening directory `%s'",
          arg);
        break;
      case BYTE_GEN:
        Errx("Error generating byte");
        break;
      case MISSING_FILE:
        Errx("Could not find file `%s'",
          arg);
        break;
      case HASH_MISMATCH:
        Warn("Hash mismatch for file `%s'",
          arg);
        break;
    }
    return;
}

/* Print warning to stderr and exit */
void
Errx(const char *fmt, ...)
{
    va_list  ap;

    /* Init */
    va_start(ap, fmt);

    /* Parse fmt */
    for(; *fmt != NULL; fmt++) {
      if(*fmt == '%') {
        fmt++;
        switch(*fmt) {
          case 'd':
            fprintf(stderr, "%d", va_arg(ap, int));
            break;
          case 'c':
            fprintf(stderr, "%c", va_arg(ap, int));
            break;
          case 's':
            fprintf(stderr, "%s", va_arg(ap, char *));
            break;
        }
      } else {
        fprintf(stderr, "%c", *fmt);
      }
    }

    /* End line */
    fprintf(stderr, "\n");

    /* Cleanup and return */
    va_end(ap);
    exit(EXIT_FAILURE);
}

/* Print warning to stderr */
void
Warn(const char *fmt, ...)
{
    va_list  ap;

    /* Init */
    va_start(ap, fmt);

    fprintf(stderr, "Warning: ");

    /* Parse fmt */
    for(; *fmt != NULL; fmt++) {
      if(*fmt == '%') {
        fmt++;
        switch(*fmt) {
          case 'd':
            fprintf(stderr, "%d", va_arg(ap, int));
            break;
          case 'c':
            fprintf(stderr, "%c", va_arg(ap, int));
            break;
          case 's':
            fprintf(stderr, "%s", va_arg(ap, char *));
            break;
        }
      } else {
        fprintf(stderr, "%c", *fmt);
      }
    }

    /* End line */
    fprintf(stderr, "\n");

    /* Cleanup and return */
    va_end(ap);
    return;
}
