/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

extern char *cwd;
extern char *tmp_dir;
extern char *prog_name;

/* Stage 1 */
/* Trim or pad fp */
void
trim(FILE **fp, int size)
{
    FILE    *tmpfp;
    bgen_t   byte;
    size_t   nbytes;

    /* We dont need to do anything
       if the user didnt specify a size */
    if(size == -1) {
      return;
    }

    /* Init variables */
    tmpfp = Tmpfile();

    /* Trim file, or just copy it to tmp fp if
       it doesnt need to be trimmed */
    for(nbytes = 0;; nbytes++) {
      /* Check if we have reached the specified size */
      if(size <= nbytes) {
        break;
      }

      /* Read byte from fp */
      if(Rbyte(&byte, *fp)) {
        Fclose(*fp);
        break;
      }

      /* Write byte to tmp fp */
      Wbyte(&byte, tmpfp);
    }

    /* If size is larger than the current file size,
       append random data to file */
    for(; nbytes < size; nbytes++) {
      /* Gen byte */
      if(genbyte(&byte)) {
        Err(BYTE_GEN, NULL);
      }

      /* Write byte */
      Wbyte(&byte, tmpfp);
    }

    Fclose(*fp);
    *fp = tmpfp;
    rewind(*fp);
    return;
}

/* Stage 2 */
/* XOR input files **in_fp and output to *out_fp */
void
XORFiles(FILE **in_fp, FILE *out_fp)
{
    int      i;
    bgen_t   in_byte,
             out_byte;

    for(i = 1;; i++) {
      /* If we're out of files,
         reset i and write in_byte */
      if(in_fp[i - 1] == NULL) {
        i = 0;
        /* Write byte */
        Wbyte(&out_byte, out_fp);
        continue;
      }

      /* Read byte */
      if(Rbyte(&in_byte, in_fp[i - 1])) {
        break;
      }

      /* XOR byte */
      if(i != 1) {
        out_byte ^= in_byte;
      } else {
        out_byte = in_byte;
      }
    }

    /* Rewind files */
    rewind(out_fp);
    for(i = 0; in_fp[i] != NULL; i++) {
      rewind(in_fp[i]);
    }

    return;
}

/* Last stage */
/* Copy file from tmpfp to outfp */
FILE *
Copy(FILE *tmpfp, char **outfn)
{
    FILE    *outfp;
    bgen_t   byte;

    if(outfn == NULL) {
      outfp = stdout;
    } else {
      outfp = tmp_fp(cwd, outfn);
    }

    for(;;) {
      /* Read byte from fp */
      if(Rbyte(&byte, tmpfp)) {
        if(outfn != NULL) {
          Fclose(tmpfp);
        }
        break;
      }

      /* Write byte to tmp fp */
      Wbyte(&byte, outfp);
    }

    rewind(tmpfp);
    if(outfn != NULL) {
      rewind(outfp);
    }
    return(outfp);
}
